/*
Cooperative PSO
 based on
 Standard PSO 2011 (from the Particle Swarm Central http://particleswarm.info)

Test of five cooperations mechanisms, plus hybrids.
                    
 Contact for remarks, suggestions etc.:
Maurice.Clerc@WriteMe.com

2012-12-20                  
	                    
*/


#include "main.h"

// =================================================
int main () 
{ 
	struct position bestBest; // Best position over all runs
	int coop1, coop2; // For hybrids
	int d;			// Current dimension
	double error;			// Current error
	int ERR;
	double errorMean;		// Average error
	double errorMin;		// Best result over all runs
	double errorMeanBest[R_max]; 
	double evalMean;		// Mean number of evaluations
	int functionCode;
	int func[funcMax]; // List of functions (codes) to optimise
	int indFunc; 

	double meanSuccess;
	int nbFunc;
	int nFailure;		// Number of unsuccessful runs
	double logProgressMean;

	// Just to test several cooperation mechanisms successively 
	// in one big run (the whole night on my laptop)
	//int mechanism[15]= {1,2,3,4,5,12,13,14,15,23,24,25,34,35,45}; int mechMax=15;
	int mechanism[5]={12,3,13,23,2}; int mechMax=5;
	// (if used, uncomment lines 278, 280, 281)
	int m;
	
	struct param param;
	struct problem pb; 
	int run, runMax; 
	struct result result; 

	int s;
	double success[funcMax];
	double successRate;
	int t;
	int test;
	double variance;
	double zz;

	E = exp ((long double) 1);  
	pi = acos ((long double) -1); 
	errMax=0;
	nbRand=0;

	// Files
	f_run = fopen ("f_run.txt", "w");  
	f_proba = fopen ("f_proba.txt", "w"); 
	f_synth = fopen ("f_synth.txt", "w");
	f_trace = fopen ("f_trace.txt", "w"); // For information

	//------------------------------------------------ PARAMETERS
	// * => suggested value

	param.S=40; //* 40. Swarm size or Mean swarm size (if BW[0]=1) 
	param.adapt=0; // 1 => use mutation-selection (automatic for Kin)
	// but has to be specified if needed for the other cooperation mechanisms
	// 0 else

	// Mutation and selection: useful only if param.adapt=1 or param.coop=3, 3n or m3
	param.adaptMutW=1./param.S;	 // * 1/S
	param.adaptMutC=param.adaptMutW;	
	param.adaptSelect=0.5; //* 0.5; // Selection rate. Must be <=0.5

	if(param.adaptSelect>0.5) param.adaptSelect=0.5; 
	//---
	param.c1 = 0.5 + log ((double) 2); //* 1.193. Confidence coefficient
	param.c2=param.c1;

	param.confin=0; 	//* 0 => keep inside the search space (supposed to be a D-rectangle)
	// 1 => no confinement 
	//   WARNING: may be very slow (and bad) for discrete problems

	param.coop=1; 	// Cooperation rules. WARNING. Useless if you use
									//                   the loop on mechanism[] 
									//  1 => Reciprocity 
									// 	2 => Vicinity (neighbours)
									//	3 => Kin (genetically similar)
									//  4 => Reputation
									// 	5 => Anybody
									//  m,n => hybrid m,n (ex.: 24 => Vicinity/Reputation)
									//  99 => cyclically 1 to 4 if no improvement

	param.distrib=0; 	// Kind of randomness in a sphere (see alea_sphere(..))
	//* 0 => uniform direction and uniform radius
	//      Note that it implies a _non_ uniform distribution 
	// 1 => Gaussian (Box-Muller method). Warning: infinite loop possible
	// 2 =>	Gaussian (CMS method)
	// 3 => Other stable (CMS, experimental parameters)
	// 4 => Slash distribution (Gaussian BM/Gaussian BM)

	// Useful only if param.distrib>0;
	param.distribMean=0.5; //* 0.5. For some functions 0 is better, though 
	param.distribSigma=1./12; //* 1./12 (standard deviation of U(0,1))	
	// WARNING: the CMS method may not work with randomness option >2
	//---	
	param.helpInit=2; //* 2
	param.helpDecay=1; //* 1

	switch(param.coop) // You may want different parameter values depending on "coop"
	{
		default:
			param.alpha=0.61; // For adaptive helpMin (param.helpOption = 2)
	param.helpMin=0; // 0 =>  for pure Reciprocity, or pure Vicinity
										// WARNING: useless if param.helpOption=2 (see below)
			break;
// Include here other cases. Be careful if using hybrids
	// And be careful if you use "mechanism[]"
	// You should then include this switch inside the 
	//		for(m=0;m<mechMax;m++)  loop
			// ........
	}
	param.helpStep=2; // * 
	param.helpUpdate=0; //* 0 => modify the help matrix at each iteration
								// 1 => only if there has been no improvement of the global best
	param.helpOption=2; // For Reciprocity and Reputation (and the hybrids that use them)
	// 0 =>(see PSO.c) (norm a)
											// 1 => (norm b)
											// 2 => norm c. WARNING: with this option, 
											//       param.helpMin is useless (adaptive helpMin)
											// WARNING; hardcoded sub-options 
	helpSubOption=0 ; //0 or 1.  See helpAdapt()
	
				// 10 => Proportional, for tests
	

	param.K=3; 	// * 3.  Parameter to compute the probability p for a particle to be an
							// external informant. You may also directly define p (see below),
							// but K is about the mean number of the informants of a particle. 
	//
	param.randCase=1; // 0 => KISS (Warning: may not work on a 64 bits machine)
										//* 1 => KISS_PRNG
									// 10 => Mersenne


	param.randNum=1;	// * 1 => random numbering of the particles before each iteration
	// 0 => always the same loop "particle 0 to S-1"

	param.reputInit=param.helpInit; 
	param.reputDecay=param.helpDecay;
	param.reputMin=param.helpMin; // WARNING: useless if param.helpOption=2
	param.reputStep=param.helpStep; 

	param.updateOption=0; // 0 => linear
												// 1 => proportional
	
	param.w = 1. / (2 * log ((double) 2)); // * 0.721. Inertia weight

	
	 //--- For sensitivity TEST
	 //param.c1=pow(param.w+1,2)/2; param.c2=param.c1; 
	 //param.w=0.4; param.c1=0.4; param.c2=0.4;
	// param.adaptMutW=1;
	// param.adaptMutC=1;
	 //---
	 
	// Check consistency
	ERR=0;
	if(param.helpOption==10)
	{
		if(param.helpStep>1)
		{
			ERR=1;
			printf("\n Sorry, with helppOption==1? helpStep must be smaller than 1");
		}

		if(param.reputStep>1)
		{
			ERR=1;
			printf("\n Sorry, with helppOption==1? reputStep must be smaller than 1");
		}

		if(param.helpDecay>1)
		{
			ERR=1;
			printf("\nHmmm, with helpOption==1, helpDecay must be smaller than 1");
		}

		if(param.reputDecay>1)
		{
			ERR=1;
			printf("\nHmmm, with helpOption==1, reputDecay must be smaller than 1");
		}

		if(param.helpStep<param.helpDecay)
		{
			ERR=1;
			printf("\n helpIncr must be greater than helpDecay");
		}

		if(param.reputStep<param.reputDecay)
		{
			ERR=1;
			printf("\n helpIncr must be greater than helpDecay");
		}
	}

	if(ERR==1) ERROR("STOP. Please check the parameter values.");
	
	// ----------------------------------------------- PROBLEM
	param.trace=0; // If >0 more information is displayed/saved (f_trace.txt)
	// Functions to optimise

	nbFunc=1; //13. Number of functions
	func[0]=102; // 4
	func[1]=-99;  // 11
	func[2]=-100;  // 15
	func[3]=-100;  // 17
	func[4]=18; // 18
	func[5]=20; // 20
	func[6]=21;  // 21
	func[7]=-100; // 100
	func[8]=102; // 102
	func[9]=-99; // 103
	func[10]=-100; // 104
	func[11]=105; // 105
	func[12]=106;	// 106	
	/*
	nbFunc=13; //13. Number of functions
	func[0]=18; // 4
	func[1]=11;  // 11
	func[2]=15;  // 15
	func[3]=17;  // 17
	func[4]=18; // 18
	func[5]=20; // 20
	func[6]=21;  // 21
	func[7]=100; // 100
	func[8]=102; // 102
	func[9]=103; // 103
	func[10]=104; // 104
	func[11]=105; // 105
	func[12]=106;	// 106	
	*/
	/* (see problemDef( ) for precise definitions)
	-100 Dummy (quick, and success rate=100)
	-99 Dummy (quick, and success rate = 0)
	-1  Constant. For test of biases
	0 Parabola (Sphere)
	1 Griewank
	2 Rosenbrock (Banana)
	3 Rastrigin
	4 Tripod (dimension 2)
	5 Ackley
	6 Schwefel
	7 Schwefel 1.2
	8 Schwefel 2.2
	9 Neumaier 3
	10 G3
	11 Network optimisation (Warning: see problemDef() and also perf() for
			problem elements (number of BTS and BSC)
	12 Schwefel
	13 2D Goldstein-Price
	14 Schaffer f6
	15 Step	
	16 Schwefel 2.21
	17 Lennard-Jones
	18 Gear train
	19 Sine_sine function
	20 Perm function
	21 Compression Spring
	22 Cellular phone (2D) 
	23 Penalized
	24 Repulsion
	25 Pressure Vessel (penalty method)
	26 Ellipsoidal
	27 Quadric
	28 Frequency modulation sound parameter identification

CEC 2005 benchmark  (no more than 30D. See cec2005data.c)
100 F1 (shifted Parabola/Sphere) 
102 F6 (shifted Rosenbrock) 
103 F9 (shifted Rastrigin) 
104 F2 Schwefel 
105 F7 Griewank  (NOT rotated)
106 F8 Ackley  (NOT rotated)

999 for tests
											*/ 

	runMax = 100; // Numbers of runs
	if (runMax > R_max) runMax = R_max;
	
//for(m=0;m<mechMax;m++) // WARNING: To comment, if mechanism[] is not used
	{
		//fprintf(f_synth,"\n%i \n",mechanism[m]);// Comment if mechanism[] is not used
		//param.coop=mechanism[m];  //Comment if mechanism[] is not used
		
	for(indFunc=0;indFunc<nbFunc;indFunc++) // Loop on problems
	{
		functionCode =func[indFunc];

		for(t=0;t<followMax;t++) followIter[t][indFunc]=0 ; // Just for information

		// Some information
		printf ("\n Function %i ", functionCode);

		// Define the problem
		pb=problemDef(functionCode);
		if(pb.SS.D>DMax) ERROR ("Can't solve it. You should increase DMax");	
	
		// ----------------------------------------------- RUNS	
		errorMean = 0;	    
		evalMean = 0;	    
		nFailure = 0;	
		if(param.coop<99) // Hybrid nm
		{
			coop1=param.coop/10;
			coop2=param.coop-10*coop1;
		}

switch(param.randCase)
		{
			case 0:
				seed_rand_kiss(1294404794); // Initialise the RNG KISS for reproducible results
				break;

				case 1: // KISS PRNG
				randk_reset();
				randk_seed();
					break;

				case 10: // Mersenne 64 bits
					init_genrand64(1294404794);
				break;	
		}

		switch(param.randCase) // "Warm up" the RNG for pseudo-random numbers
		{
			case 1:
				randk_warmup(10000);
				break;

			default:
				for (t=0;t<10000;t++) zz=alea(0,1,param.randCase); 
				break;
		}
		//*************
		//for(test=0;test<100;test++) // Uncomment for distribution of the success rate
																// The result will be in f_synth.txt
		{ nFailure=0;
			for (run = 0; run < runMax; run++)  
			{	
				// Save information (useful only if using Kin
				fprintf(f_trace,"\n%i ",run+1);
				fprintf(f_trace,"\nIter ");
				for(s=0;s<param.S;s++)				fprintf(f_trace,"%i ", s ); // For w
				fprintf(f_trace,"Iter ");
				for(s=0;s<param.S;s++)				fprintf(f_trace,"%i ", s ); // For c1
				fprintf(f_trace,"Iter ");
				for(s=0;s<param.S;s++)				fprintf(f_trace,"%i ", s ); // For c2
				fprintf(f_trace,"\n");

				// Compute the probability to be informed
				param.p=1-pow(1-1./param.S,param.K);     

				result = PSO (param, pb,indFunc);
				error = result.error;

				if (error > pb.epsilon) // Failure
					nFailure = nFailure + 1;

				if(pb.SS.normalise>0)
				{
					for(d=0;d<pb.SS.D;d++)
						result.SW.P[result.SW.best].x[d]=
						pb.SS.min[d]+(pb.SS.max[d]-pb.SS.min[d])*result.SW.P[result.SW.best].x[d]
						/pb.SS.normalise;
				}		

				// Memorize the best (useful if more than one run)
				if(run==0) bestBest=result.SW.P[result.SW.best];
				else
					if(error<bestBest.f) bestBest=result.SW.P[result.SW.best];

				// Result display
				errorMean=errorMean+error;

				//	if(run==runMax-1) //***
				{
					printf ("\nRun %i. S %i,  Eval %f. Error %e ", run+1, param.S, result.nEval, error);
					printf(" Mean %e",errorMean/(run+1));
					zz=100*(1-(double)nFailure/(run+1));
					printf("  Success  %.2f%%",zz);
					printf(" best particle %i", result.SW.best);

					// Best position display
					//for (d=0;d<pb.SS.D;d++) printf(" %f",result.SW.P[result.SW.best].x[d]);

					// Save result
					fprintf( f_run, "\n%i %.1f %.0f %e %e ", run+1, zz, result.nEval,  error, errorMean/(run+1) );
					// Save best position
					for ( d = 0; d < pb.SS.D; d++ ) fprintf( f_run, " %f",  result.SW.P[result.SW.best].x[d] );
				}
				fprintf(f_trace,"best %i",result.SW.best);

				// Compute/save some statistical information
				if (run == 0)
					errorMin = error;
				else if (error < errorMin)
					errorMin = error;

				evalMean = evalMean + result.nEval;	
				errorMeanBest[run] = error;
				logProgressMean  = logProgressMean - log(error);		
			}		// End loop on "run"

			// ---------------------END 

			// Display some statistical information
			evalMean = evalMean / (double) runMax;   
			errorMean = errorMean / (double) runMax;
			logProgressMean = logProgressMean/(double) runMax;

			printf ("\n Eval. (mean)= %f", evalMean);	
			printf ("\n Error (mean) = %e", errorMean);

			// Variance
			variance = 0;

			for (run = 0; run < runMax; run++)
				variance = variance + pow (errorMeanBest[run] - errorMean, 2);

			variance = sqrt (variance / runMax);	    
			printf ("\n Std. dev. %e", variance); 
			printf("\n Log_progress (mean) = %f", logProgressMean);	

			// Success rate and minimum value
			printf("\n Failure(s) %i  ",nFailure);
			successRate = 100 * (1 - nFailure / (double) runMax);			
			printf ("Success rate = %.2f%%", successRate);
			success[indFunc]=successRate;

			printf ("\n Best min value = %1.20e", errorMin);
			printf ("\nPosition of the optimum: ");
			for (d=0;d<pb.SS.D;d++) printf(" %.20f",bestBest.x[d]);


			// Save	
			fprintf (f_synth,"%i %i %.0f %.20f %.20f %f %f", functionCode, pb.SS.D,successRate,
			         errorMean, variance, evalMean,bestBest.f);
			for (d=0;d<pb.SS.D;d++) fprintf(f_synth, " %1.20e",bestBest.x[d]);  
			fprintf(f_synth,"\n"); 

			// Specific save for Repulsive problem
			if(pb.function==24)
			{		
				for (d=0;d<pb.SS.D-1;d=d+2) 
				{
					fprintf(f_synth, " %1.20e %1.20e",bestBest.x[d],bestBest.x[d+1]);  
					fprintf(f_synth,"\n"); 
				}	
			}	     	   
		} //*** test
	} // End "for ind[..."

	printf("\n errMax : %f",errMax);

	// Repeat informations
	printf("\n---------");
	printf("\n Function(s):");
	zz=0;
	for(indFunc=0;indFunc<nbFunc;indFunc++) // Loop on problems
	{
		zz=zz+success[indFunc];
		functionCode =func[indFunc];
		printf(" %i",functionCode);
	}
	meanSuccess=zz/nbFunc;
	printf("\n Mean success rate= %f",meanSuccess);
	fprintf(f_synth,". . %f",meanSuccess);
	printf("\n Confinement: "); 
	if(param.confin==0) printf("YES"); else printf("NO");
	printf("\n Distribution: ");
	switch(param.distrib)
	{
		case 0: printf(" uniform"); break;
		case 1: printf(" Gaussian (%f,%f), Box-Muller",param.distribMean,param.distribSigma); break;
		case 2: printf(" Gaussian (%f,%f), CMS",param.distribMean,param.distribSigma); break;
		case 3: printf(" Stable (%f,%f)",param.distribMean,param.distribSigma); break;
		case 4: printf(" Slash (%f,%f)",param.distribMean,param.distribSigma); break;
	}

	printf("\n Swarm size: %i", param.S);
	printf("\n K = %i",param.K);
	printf("\n w = %f",param.w);
	printf("\n c1, c2 = %f %f",param.c1, param.c2);
	if (param.adapt==1 || coop1==3 || coop2==3) printf("\n WARNING: adaptation of w and c");
	printf("\n Mutation w = %f",param.adaptMutW);
	printf("\n Mutation c = %f",param.adaptMutC);
	printf("\n Selection  = %f", param.adaptSelect);
	printf("\n Cooperation: %i", param.coop);
	printf("\n helpOption: %i",param.helpOption);

	if(param.helpOption==2) printf(" => adaptive helpMin");
	else printf(" with helpMin = %f", param.helpMin);
	                          
	printf("\n %e random numbers have been used",nbRand);
	fprintf(f_run,"\nnbRand %e",nbRand);

	// Just for information about the variable topology
	for(indFunc=0;indFunc<nbFunc;indFunc++)
	{
		zz=followIter[0][indFunc]; // Should be zero, anyway
		for(t=1;t<followMax;t++) zz=followIter[t][indFunc]+zz; // Cumul
		if(zz<0) zz=1;

		for(t=1;t<followMax;t++)
		{
			followIter[t][indFunc]=followIter[t][indFunc]/zz; // Probability distribution
		}
	}

	for(t=1;t<followMax;t++)
	{
		fprintf(f_proba,"%i ",t);
		for(indFunc=0;indFunc<nbFunc;indFunc++)
			fprintf(f_proba,"%f ",followIter[t][indFunc]);
		fprintf(f_proba,"\n");
	}

} // end of loop on mechanisms
	return 0; // End of main program
}
// ===============================================================
#include "alea.c"
#include "perf.c"
#include "problemDef.c"
#include "PSO.c"
#include "tools.c"


